unit dspackfrm;

interface

{$I dx_define.inc}

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, DSPack, dx_vgcore, dx_scene, dx_vglayer, DirectShow9,
  {$IFDEF KS_COMPILER10_UP} DSUtils, {$ELSE} DSUtil,  {$ENDIF}
  dx_objects, dx_utils;

type
  TfrmDSPack = class(TForm)
    dxScene1: TdxScene;
    Root1: TdxVisualObject;
    Toolbar: TdxGUIObjectLayer;
    Root2: TvxLayout;
    ToolBar1: TvxToolBar;
    Button1: TvxButton;
    FilterGraph: TFilterGraph;
    VideoWindow1: TVideoWindow;
    SampleGrabber1: TSampleGrabber;
    OpenDialog: TOpenDialog;
    Timer1: TTimer;
    Camera1: TdxCamera;
    Image1: TdxImage;
    Dummy1: TdxDummy;
    Dummy2: TdxDummy;
    FrameLayer: TdxObjectLayer;
    Root3: TvxLayout;
    Rectangle1: TvxRectangle;
    PositionTrack: TvxTrack;
    GlowEffect1: TvxGlowEffect;
    Rectangle2: TvxRectangle;
    VolumeBar: TvxTrackBar;
    Text1: TvxText;
    procedure Button1Click(Sender: TObject);
    procedure SampleGrabber1Buffer(sender: TObject; SampleTime: Double;
      pBuffer: Pointer; BufferLen: Integer);
    procedure Root1MouseMove(Sender: TObject; Shift: TShiftState; X, Y: Single; rayPos, rayDir: TdxVector);
    procedure Root1MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Single; rayPos, rayDir: TdxVector);
    procedure Root1MouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Single; rayPos, rayDir: TdxVector);
    procedure Root1MouseWheel(Sender: TObject; Shift: TShiftState;
      WheelDelta: Integer; MousePos: TdxPoint; var Handled: Boolean);
    procedure Timer1Timer(Sender: TObject);
    procedure PositionTrackChange(Sender: TObject);
    procedure VolumeBarChange(Sender: TObject);
  private
    { Private declarations }
    Down: TdxPoint;
    Pressed: boolean;
  public
    { Public declarations }
  end;

var
  frmDSPack: TfrmDSPack;

implementation

{$R *.dfm}

procedure TfrmDSPack.Button1Click(Sender: TObject);
begin
  if OpenDialog.Execute then
  begin
    if not FilterGraph.Active then FilterGraph.Active := true;
    // clear bitmap
    Image1.Bitmap.SetSize(1, 1);
    //
    FilterGraph.ClearGraph;
    try
      FilterGraph.RenderFile(OpenDialog.FileName);
      VolumeBar.Value := FilterGraph.Volume / 10000;
      FilterGraph.Play;
    except
    end;
  end;
end;

procedure TfrmDSPack.SampleGrabber1Buffer(sender: TObject;
  SampleTime: Double; pBuffer: Pointer; BufferLen: Integer);
var
  i: integer;
  B: Tbitmap;
  Bits: PdxColorArray;
begin
  { skip transfer if scene is drawing }
  if dxScene1.IsDrawing then Exit;
  { grab stream to TvgBitmap }
  if Image1.Bitmap.Width = 1 then
  begin
    { First time we need to get video size }
    B := Tbitmap.create;
    SampleGrabber1.GetBitmap(B);
    Image1.Bitmap.SetSize(B.Width, B.Height);
    Image1.Width := B.Width / 100;
    Image1.Depth := B.Height / 100;
    // Just a frame
    FrameLayer.Width := (B.Width / 100) + 0.85;
    FrameLayer.Depth := (B.Height / 100) + 0.85;
    B.free;
  end;
  { grabbing }
  if Image1.Bitmap.Width > 1 then
  begin
    if Image1.Bitmap.LockBitmapBits(Bits, true) then
    begin
      try
        // transfer - need to flip - because source have different orientation
        for i := 0 to Image1.Bitmap.Height - 1 do
          Move(PdxColorArray(pBuffer)[(Image1.Bitmap.Height - i - 1) * Image1.Bitmap.Width], Bits[i * Image1.Bitmap.Width], Image1.Bitmap.Width * 4);
        // set alpha-channel
        vgFillAlpha(Bits, BufferLen div 4, $FF);
      finally
        Image1.Bitmap.UnlockBitmapBits;
      end;
    end;
  end;
end;

procedure TfrmDSPack.Root1MouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Single; rayPos, rayDir: TdxVector);
begin
  if Pressed and (ssLeft in Shift) then
  begin
    Dummy1.RotateAngle.Z := Dummy1.RotateAngle.Z + (X - Down.X);
    Dummy2.RotateAngle.X := Dummy2.RotateAngle.X + (Y - Down.Y);
    Down := dxPoint(X, Y, 0);
  end;
end;

procedure TfrmDSPack.Root1MouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Single; rayPos, rayDir: TdxVector);
begin
  Root1.AutoCapture := true;
  Pressed := true;
  Down := dxPoint(X, Y, 0);
end;

procedure TfrmDSPack.Root1MouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Single; rayPos, rayDir: TdxVector);
begin
  Pressed := false;
end;

procedure TfrmDSPack.Root1MouseWheel(Sender: TObject; Shift: TShiftState;
  WheelDelta: Integer; MousePos: TdxPoint; var Handled: Boolean);
begin
  Camera1.Position.Vector := dxVectorAdd(Camera1.Position.Vector, dxVectorScale(dxVector(0, 1, 0),
    (WheelDelta / 120) * 0.3));
end;

procedure TfrmDSPack.Timer1Timer(Sender: TObject);
var
  MediaSeeking: IMediaSeeking;
  CurrentPos, StopPos: int64;
  MlsCurrentPos, MlsStopPos: Cardinal;
begin
  if FilterGraph.Active and (not PositionTrack.IsTracking) then
  begin
    FilterGraph.QueryInterface(IMediaSeeking, MediaSeeking);
    if Assigned(MediaSeeking) then
    begin
      if Succeeded(MediaSeeking.GetDuration(StopPos)) then
      if Succeeded(MediaSeeking.GetCurrentPosition(CurrentPos)) then
      begin
        MlsCurrentPos := RefTimeToMiliSec(CurrentPos);
        MlsStopPos    := RefTimeToMiliSec(StopPos);

        PositionTrack.OnChange := nil; // disable on change
        PositionTrack.min := 0;
        PositionTrack.max := MlsStopPos;
        PositionTrack.value := MlsCurrentPos;
        PositionTrack.Viewportsize := MlsStopPos / 10;
        PositionTrack.OnChange := PositionTrackChange;
      end;
    end;
    MediaSeeking := nil;
  end;
end;

procedure TfrmDSPack.PositionTrackChange(Sender: TObject);
var
  MediaSeeking: IMediaSeeking;
  CurrentPos, StopPos: int64;
  MlsCurrentPos, MlsStopPos: Cardinal;
begin
  if FilterGraph.Active then
  begin
    FilterGraph.QueryInterface(IMediaSeeking, MediaSeeking);
    if Assigned(MediaSeeking) then
    begin
      if Succeeded(MediaSeeking.GetDuration(StopPos)) then
      if Succeeded(MediaSeeking.GetCurrentPosition(CurrentPos)) then
      begin
        CurrentPos := (StopPos * trunc(PositionTrack.Value)) div trunc(PositionTrack.max);
        MediaSeeking.SetPositions(CurrentPos, AM_SEEKING_AbsolutePositioning,
          StopPos, AM_SEEKING_NoPositioning);
      end;
    end;
    MediaSeeking := nil;
  end;
end;

procedure TfrmDSPack.VolumeBarChange(Sender: TObject);
begin
  FilterGraph.Volume := Round(VolumeBar.Value * 10000);
end;

end.
